@@ -859,7 +859,7 @@ populate_date(EContactField field, EContact *new_contact, void *data)
 
 	dt.year = then->tm_year + 1900;
 	dt.month = then->tm_mon + 1;
-	dt.day = then->tm_mday;
+	dt.day = then->tm_mday + 1;
 	
 	e_contact_set (new_contact, field, &dt);
 }
@@ -961,14 +961,16 @@ proppatch_address (PropMapping *prop_mapping,
 			continue;
 		}
 
+		/* If a contact is being modified, we need to also check if the extension field has changed*/
 		if (cur_address && cur_addrprops[i] &&
-		    !strcmp (new_addrprops[i], cur_addrprops[i])) {
+		    !strcmp (new_addrprops[i], cur_addrprops[i]) &&
+		    !strcmp (new_address->ext, cur_address->ext)) {
 			/* they're the same */
 			continue;
 		}
 
 		if (i == ADDRPROP_STREET && new_address->ext) {
-			value = g_strdup_printf ("%s%s", new_addrprops[i],
+			value = g_strdup_printf ("%s %s", new_addrprops[i],
 						 new_address->ext);
 		} else
 			value = g_strdup (new_addrprops[i]);
