@@ -1071,6 +1071,27 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		g_free (reninfo.old_base);
 		break;
 	}
+	
+	case CAMEL_STUB_RETVAL_FOLDER_SET_READONLY:
+	{
+		CamelFolder *folder;
+		gchar *folder_name;
+		guint32 readonly;
+
+		if (camel_stub_marshal_decode_folder (stub->status, &folder_name) == -1 ||
+		    camel_stub_marshal_decode_uint32 (stub->status, &readonly) == -1)
+			break;
+
+		g_mutex_lock (exch->folders_lock);
+		folder = g_hash_table_lookup (exch->folders, folder_name);
+		g_mutex_unlock (exch->folders_lock);
+		if (folder) {
+			camel_exchange_summary_set_readonly (folder->summary, readonly ? TRUE : FALSE);
+		}
+
+		g_free (folder_name);
+		break;
+	}
 
 	default:
 		g_assert_not_reached ();
