@@ -1446,7 +1446,7 @@ append_message (MailStub *stub, const char *folder_name, guint32 flags,
 	MailStubExchange *mse = MAIL_STUB_EXCHANGE (stub);
 	MailStubExchangeFolder *mfld;
 	E2kHTTPStatus status;
-	char *ru_header, *repl_uid, *location;
+	char *ru_header = NULL, *repl_uid, *location = NULL;
 
 	mfld = folder_from_name (mse, folder_name, MAPI_ACCESS_CREATE_CONTENTS, FALSE);
 	if (!mfld)
@@ -2665,6 +2665,12 @@ subscribe_folder (MailStub *stub, const char *folder_name)
 		return;
 	}
 	
+	if (!strcmp (e_folder_get_type_string (folder), "noselect")) {
+		g_object_unref (folder);
+		mail_stub_return_ok (stub);
+		return;
+	}
+	
 	result = exchange_account_add_favorite (mse->account, folder);
 
 	switch (result) {
@@ -2716,8 +2722,8 @@ unsubscribe_folder (MailStub *stub, const char *folder_name)
 	}
 
 	g_object_unref (folder);
-	
-	pub_name = strchr (folder_name, '/');
+
+	pub_name = strrchr (folder_name, '/');
 	path = g_build_filename ("/favorites", pub_name, NULL);
 	folder = exchange_account_get_folder (mse->account, path);
 	if (!folder) {
